import java.io.File;
import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class PlantBuilder
{
	public static final String SCHEMAFILE = "ProductionPlant.xsd";
	public static Document doc;

	public static void main(String[] args)
	{
		try 
		{
			readXMLFile("SamplePlant.xml");
		}
		catch (Exception e)
		{
			System.out.println(e);
		}
	}

	private static void readXMLFile(String xmlFile) throws Exception
	{
		// create a factory that understands namespaces and validates the XML input
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);

		try
		{
			// read the XML file
			DocumentBuilder builder = factory.newDocumentBuilder();
			builder.setErrorHandler(new PPErrorHandler());
			doc = builder.parse(new File(xmlFile));

			// create a SchemaFactory and a Schema
			SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			Source schemaFile = new StreamSource(new File(PlantBuilder.SCHEMAFILE));
			Schema schema = schemaFactory.newSchema(schemaFile);

			// create a Validator object and validate the XML file
			Validator validator = schema.newValidator();
			validator.validate(new DOMSource(doc));
			System.out.println("XML file successfully validated.");
		}
		catch (SAXParseException spe)
		{
			// errror generated by the parser
			String message = "** Parser error! **\n  URI: " + spe.getSystemId() + "\n  Line: " + spe.getLineNumber() + "\n  Message: " + spe.getMessage();
			Exception x = spe;
			if (spe.getException() != null)
				x = spe.getException();
			throw new Exception(message, x);
		}
		catch (SAXException sxe)
		{
			Exception x = sxe;
			if (sxe.getException() != null)
				x = sxe.getException();
			throw new Exception("Error during parsing.", x);

		}
		catch (ParserConfigurationException pce)
		{
			throw new Exception("Parser with specified options can't be built.", pce);
		}
		catch (IOException ioe)
		{
			// I/O error
			throw new Exception("Error reading file.", ioe);
		}
	}
}